<#ibiztemplate>
TARGET=PSSYSTEM
</#ibiztemplate>
<#if sys.getCodeName()=='Sample'>
package ${pub.getPKGCodeName()}.util.service;

import lombok.extern.slf4j.Slf4j;
import ${pub.getPKGCodeName()}.util.client.IBZLiteFeignClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.io.*;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class DynamicModelService {

    @Value("${r'$'}{ibiz.systemid:${sys.getCodeName()}}")
    private String systemId;

    @Value("${r'${ibiz.dynamic.path:/app/file/dynamicmodel/}'}")
    private String dynamicPath;

    @Autowired
    IBZLiteFeignClient liteClient;

    /**
     * 同步动态模型
     * @return
     */
    public boolean syncModel() {
        log.debug("开始同步动态模型");
        long start = System.currentTimeMillis();
        List<Map<String, Object>> dynamicModels = liteClient.getDynamicModel(systemId);
        if (!ObjectUtils.isEmpty(dynamicModels)) {
            File dynamicFolder = new File(dynamicPath + File.separator + systemId);
            if (dynamicFolder.exists()) {
                deleteDir(dynamicFolder.getPath());
            }
            for (Map<String, Object> dynamicModel : dynamicModels) {
                InputStream inputStream = null;
                BufferedInputStream bufferedInputStream = null;
                FileOutputStream outputStream = null;
                BufferedOutputStream bufferedOutputStream = null;
                try {
                    String strFileName = String.valueOf(dynamicModel.get("filename"));
                    String strFilePath = String.valueOf(dynamicModel.get("filepath"));
                    Object strFileContent = dynamicModel.get("filecontent");
                    String fullPath = dynamicPath + strFilePath;
                    File parent = new File(fullPath.replace(strFileName, ""));
                    if (!parent.exists()) {
                        parent.mkdirs();
                    }
                    File file = new File(fullPath);
                    file.createNewFile();
                    // 文件输入流并进行缓冲
                    inputStream = new ByteArrayInputStream(String.valueOf(strFileContent).getBytes());
                    bufferedInputStream = new BufferedInputStream(inputStream);
                    // 文件输出流并进行缓冲
                    outputStream = new FileOutputStream(file);
                    bufferedOutputStream = new BufferedOutputStream(outputStream);
                    // 缓冲数组，大文件 可将 1024 * 2 改大一些，但是 并不是越大就越快
                    byte[] bytes = new byte[1024 * 2];
                    int len = 0;
                    while ((len = inputStream.read(bytes)) != -1) {
                        bufferedOutputStream.write(bytes, 0, len);
                    }
                    // 刷新输出缓冲流
                    bufferedOutputStream.flush();
                    log.debug(fullPath);
                } catch (IOException e) {
                } finally {
                    try {
                        if (bufferedInputStream != null) {
                            bufferedInputStream.close();
                        }
                        if (bufferedOutputStream != null) {
                            bufferedOutputStream.close();
                        }
                        if (inputStream != null) {
                            inputStream.close();
                        }
                        if (outputStream != null) {
                            outputStream.close();
                        }
                    } catch (IOException e) {
                    }
                }
            }

        }
        long end = System.currentTimeMillis();
        log.debug("动态模型同步完成，共耗时"+ (end - start) / 1000 + " 秒");
        return true;
    }

    /**
     * 删除历史动态模型
     * @param dirPath
     */
    public void deleteDir(String dirPath) {
        File file = new File(dirPath);
        if (file.isFile()) {
            file.delete();
        } else {
            File[] files = file.listFiles();
            if (files == null) {
                file.delete();
            } else {
                for (int i = 0; i < files.length; i++) {
                    deleteDir(files[i].getAbsolutePath());
                }
                file.delete();
            }
        }
    }
}
</#if>